subroutine CalcEnergyForces(Pos, Plates, q_plate, L, rc, PEnergy, Forces, Dim, NIon,NPlate)
    implicit none
    integer, intent(in) :: Dim, NIon, NPlate
    real(8), intent(in), dimension(0:NIon-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NPlate-1, 0:Dim-1) :: Plates
    real(8), intent(in) :: q_plate, L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NIon-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: Debye = 5.
    real(8), dimension(Dim) :: rij, Fij, Posi, riw, Fiw
    real(8) :: d2, id2, id6, id12, dw2
    real(8) :: rc2, Shift, rij_scalar, riw_scalar
    integer :: i, j, w
    !Compute the Lennard-Jones shift value before the pairwise loop
    Shift = -4. * (rc**(-12))
    rc2 = rc * rc
    PEnergy = 0.
    Forces = 0.
    do i = 0, NIon - 1
       	
	!store Pos(i,:) in a temporary array for faster access in j and w loop
        Posi = Pos(i,:)

	!Calc ION-WALL PE_electric and F_electric & PE_LJ and F_LJ	   
	do w = 0, NPlate - 1
 	   riw=Plates(w,:) - Posi
	   riw(:2) = riw(:2) - L * dnint(riw(:2) / L)
	   dw2=sum(riw*riw)
	   riw_scalar=sqrt(dw2)

	   !ION-WALL PE_electric and F_electric	
           if (q_plate /= 0) then				
	   	if (mod(i,2)+mod(w,2) == 1) then !Only possible for oppositely charged species
	        	PEnergy=PEnergy-q_plate*exp(-riw_scalar/Debye)/riw_scalar
	   		Fiw=-riw*q_plate*exp(-riw_scalar/Debye)/riw_scalar**3.
		        Forces(i,:) = Forces(i,:) + Fiw
		else
	   	        PEnergy=PEnergy+q_plate*exp(-riw_scalar/Debye)/riw_scalar
	   	        Fiw=riw*q_plate*exp(-riw_scalar/Debye)/riw_scalar**3.
	  	        Forces(i,:) = Forces(i,:) + Fiw
		end if
	   end if

	   !ION-WALL PE_LJ and F_LJ
	   if (dw2 <= rc2) then
	   	id2 = 1. / dw2	   !inverse squared distance
       		id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
	        PEnergy = PEnergy + 4. * (id12) + Shift
                Fiw = riw * ((-48. * (id12)) * id2)
                Forces(i,:) = Forces(i,:) + Fiw
	    end if	
	 enddo

	!Calc ION-ION PE_electric and F_electric & PE_LJ and F_LJ
        do j = i + 1, NIon - 1
            rij = Pos(j,:) - Posi
	    rij(:2)= rij(:2) - L * dnint(rij(:2) / L)
	    d2 = sum(rij * rij)
	    rij_scalar=sqrt(d2)    

	!ION-ION PE_electric & F_electric:
	!positive unit charge for even indicies, negative for odd

	    if (mod(i,2)+mod(j,2) == 1) then !Only possible for oppositely charged ions
		PEnergy=PEnergy-exp(-rij_scalar/Debye)/rij_scalar
		Fij=-rij*exp(-rij_scalar/Debye)/rij_scalar**3.
		Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij   
	    else
		PEnergy=PEnergy+exp(-rij_scalar/Debye)/rij_scalar
		Fij=rij*exp(-rij_scalar/Debye)/rij_scalar**3.
		Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij   
	    end if

	    !ION-ION PE_LJ and F_LJ
	    if (d2 <= rc2) then
        	id2 = 1. / d2            !inverse squared distance
                id6 = id2 * id2 * id2    !inverse sixth distance
                id12 = id6 * id6         !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12) + Shift
                Fij = rij * ((-48. * (id12)) * id2)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
	    end if
    	enddo
    enddo
end subroutine

subroutine VVIntegrate(Pos, Plates, Vel, Accel, q_plate, L, rc, dt, KEnergy, PEnergy, Dim, NIon, NPlate)
    implicit none
    integer, intent(in) :: Dim, NIon, NPlate
    real(8), intent(in) :: q_plate,L, rc, dt
    real(8), intent(in), dimension(0:NPlate-1, 0:Dim-1) :: Plates
    real(8), intent(inout), dimension(0:NIon-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in, out, inplace) :: Pos,Vel,Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, Plates, q_plate, L, rc, PEnergy, Accel, Dim, NIon, NPlate)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine